#!/usr/bin/env python
# 
#   Copyright (C) 2010    John Lumby    johnlumby@hotmail.com
#     This program is based on the GTK tutorial at
#          http://library.gnome.org/devel/gtk-tutorial/
#     and several sample programs provided by the
#     Gtk and Gnome organizations,  www.gtk.org, www.gnome.org,
#     who may also hold copyrights:
#     You should consult the current reference documentation available at
#               http://www.gtk.org/documentation.html
#               http://library.gnome.org/devel/references
#     for details of how to write programs such as this.
# 
# -------------------------------------------------------------------------*
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
# 
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
# 
#   The GNU General Public License, version 2, is viewable at
#               http://www.gnu.org/licenses/gpl-2.0.html
#   The following is a brief summary of specific points from the license
#   that apply to use of this program,  but not a replacement of that license.
#   Permission is granted to use this program for any legal purpose and to copy,
#   modify, or redistribute the program subject to the terms and restrictions
#   stated in the GNU General Public License, version 2
#   and specifically provided that you abide by both of the following
#   as well as that license :
#    .   You may not incorporate this program into a proprietary program either in part
#        or in whole,  either in source or compiled form,   proprietary meaning
#        either that you claim ownership of the program or that you sell it
#        for a fee or both.
#    .   You must retain the above copyright statement and this entire notice
#        in any copy or redistribution,  and make it clear that the licensing terms,
#        restrictions and obligations apply to whomever you give or redistribute
#        this program to, including this requirement to pass on the same terms,
#        restrictions and obligations to that recipient.
# -------------------------------------------------------------------------*
#
""" Light Switch

GtkSwitch provides an on/off mechanism. 
The user can control which state should be active
by clicking the empty area or by dragging the handle.
"""

import sys
import os.path

import string
import re

#  print("PYTHON PATH:")
#  for folder in sys.path:
#      print(folder)
from gi.repository import Gdk, Gtk, GObject


class LightSwitch(Gtk.Window):
    def __init__(self, parent=None):
        self.mywindow = Gtk.Window.__init__(self)

        self.connect('delete-event', self._on_delete_event)
        
        self.connect("destroy", self.destroy)

        Gtk.Window.set_border_width(self,10)

        self.vbox = Gtk.VBox()
        self.vbox.set_spacing(85)
        self.add(self.vbox)
        self.vbox.set_border_width(8)
        self.vbox.show()

        self.light_switch = Gtk.Switch()

        self.vbox.pack_start(self.light_switch, True, True, 0)

        self.light_image = Gtk.Image()
        self.light_image.set_from_file("light_on.png")
        self.vbox.pack_end(self.light_image, True, True, 0)
        self.light_image.show()

        self.light_switch.set_active(True)
        self.light_switch.connect('notify::active', self.turn_light_on_off ,  self.light_image)
        self.light_switch.show()

        self.statusbar = Gtk.Statusbar()
        self.vbox.pack_end(self.statusbar, True, True, 0)
        self.statusbar.show()

        self.show_all()

    def turn_light_on_off(self, paramspec , maybe , this_light_image):

        on_off = self.light_switch.get_active()
        this_light_image.set_sensitive(on_off)

    def main(self):
        self.show()
        Gtk.main()

    def delete_event(self, widget, event, data=None):
# If you return FALSE in the "delete_event" signal handler,
# GTK will emit the "destroy" signal. Returning TRUE means
# you don't want the window to be destroyed.
# This is useful for popping up 'are you sure you want to quit?'
# type dialogs.
        print("delete event occurred")
# Change FALSE to TRUE and the main window will not be destroyed
# with a "delete_event".
        return False

    def _on_delete_event(self, window, event):
        self.quit()

    def destroy(self, widget, data=None):
        Gtk.main_quit()

    def quit(self):
        Gtk.main_quit()


if __name__ == '__main__':
    instance = LightSwitch()
    instance.main()
