#/usr/bin/python
"""
A simple game of  whacking simple things with a hammer
"""


#import modules
import os, pygame
from random import choice

#define the classes for our game objects
class Hammer(pygame.sprite.Sprite):
    '''controls the hammer sprite. This class is derived from the Pygame Sprite class'''
    def __init__(self):
        # we MUST call the Sprite init method on ourself
        pygame.sprite.Sprite.__init__(self) 
        #load the image resource
        self.imagesource=os.path.join('resources',  'hammer1.png')
        self.image= pygame.image.load(self.imagesource)
        # get the rectangle describing the loaded image
        self.rect=self.image.get_rect()
        #set the transparency colour to the background of the image
        self.image.set_colorkey(self.image.get_at((0, 0)), pygame.RLEACCEL)
        #copy the base image into a backup for restoring
        self.default=self.image
        self.default_rect=self.rect
        # define a flag sowe can tell what mode the hammer is in
        self.whack = 0
        # load a sound effect
        self.whacksound= pygame.mixer.Sound('resources/whack.wav')

    def update(self):
        '''The update method is called before redrawing the sprite each time
        It's our chance to control any time-related behaviour and check status.'''
        #move hammer to current mouse position
        pos = pygame.mouse.get_pos()
        self.rect.center = pos
        #check if we are supposed to be hitting something
        if self.whack:
            #if so, we move the sprite a little, and rotate the image 
            self.rect.move_ip(8, 12)
            self.image = pygame.transform.rotate(self.default, 45)


    def dowhack(self, target):
        "checks to see if hammer has hit a mike"
        if not self.whack:
            self.whack = 1
            hitrect = self.rect.inflate(-50, -50)
            return hitrect.colliderect(target.rect)

    def reset(self):
        '''put the hammer upright again and reset te flag''' 
        self.whack = 0
        self.image=self.default
    
    def playsound(self):
        '''plays the sound effect attached to the sprite'''
        self.whacksound.play()



class Mike(pygame.sprite.Sprite):
    """implements a Mike"""
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.imagesource=os.path.join('resources',  'Mikesmall.png')
        self.image= pygame.image.load(self.imagesource)
        self.rect=self.image.get_rect()
        self.image.set_colorkey(self.image.get_at((0, 0)), pygame.RLEACCEL)
        self.default=self.image
        self.default_rect=self.rect
        self.owwsound = pygame.mixer.Sound('resources/oww.wav')
        # timer of 0 is default and means the Mike is not visible
        self.timer=0
        # if whacked = 0 it means nobody has hit this Mike
        self.whacked=0
        # a list of the possible locations for Mike to pop up at
        self.locations=[(20, 300), (155, 150), (290, 300),(425, 150),  (600, 300)]
        
    def appear(self):
        '''pops out of hole'''
        # set the timer
        #if you change this value, you will also have to alter the update method
        self.timer=80
        #use the Randome.choice() function to select a location from the list
        self.loc=choice(self.locations)

    def update(self):
        '''run housekeeping'''
        #check to see that a Mike should be on te screen
        if self.timer:
            # the Mike exists and is visible
            # decrement counter to count down
            self.timer -= 1
            if self.timer > 60:
                #appearing
                xpos, ypos=self.loc
                height=self.default.get_height()
                #workout what fraction of the image we should draw
                y=(height/20)*(self.timer-60)
                #copy that surface from the backup to the main sprite image
                self.image=self.default.subsurface(0,0,self.default.get_width(),height-y)
                self.rect.left=xpos
                #adjustthe Y position so it pops up
                self.rect.top=ypos+y
            if self.timer == 0:
                #finished - disappear it
                self.rect=self.default_rect
            if self.timer < 12:
                #vanishing
                # like appearing, butin reverse
                xpos, ypos=self.loc
                height=self.default.get_height()
                y=(height/12)*self.timer
                self.image=self.default.subsurface(0,0,self.default.get_width(),y)
                self.rect.left=xpos
                self.rect.top=ypos-y+height 
            if self.whacked:
                #make him suffer
                self.whacked += 1
                #if he is hit, we can use the flag as a counter and spin the sprite through 360 degrees
                if self.whacked >= 10:
                    self.whacked = 0
                    self.image = self.default
                else:
                    self.image =pygame.transform.rotate(self.default, self.whacked*36)
                
                
    def is_hit(self):
        """things todo when object is hit"""
        self.whacked=1
        #make Mike squeal
        self.owwsound.play()
    


def main():
    """the main game logic"""
#Initialize Everything
    pygame.init()
    screensize=(800, 600)
    screen = pygame.display.set_mode(screensize)
    pygame.display.set_caption('Whack a Mike')
    pygame.mouse.set_visible(0)
    bgsource=os.path.join('resources',  'background.png')
    background = pygame.image.load(bgsource)
    background = background.convert()
    # set up a controlling timer for the game
    clock = pygame.time.Clock()
    hammer = Hammer()
    mike=Mike()
    #add the sprites to a rendering group
    spritegroup= pygame.sprite.RenderPlain(( mike,  hammer))

    # control loop
   
    while 1:
        #adjust this timer to make the game harder or easier
        clock.tick(130)
        
        #check what events Pygame has caught
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                return
            elif event.type == pygame.MOUSEBUTTONDOWN:
                #button means the hammer is being used
                hammer.playsound()
                if hammer.dowhack(mike):
                    mike.is_hit()
            elif event.type is pygame.MOUSEBUTTONUP:
                hammer.reset()
    #does a mike exist? if not, we should have one
        if mike.timer==0:
            mike.appear()

    #refresh the screen by drawing everything again
        #call the update methods of all the sprites in the group
        spritegroup.update()
        #redrawthe background
        screen.blit(background, (0, 0))
        #draw everything
        spritegroup.draw(screen)
        #flip the buffer
        pygame.display.flip()


if __name__ == '__main__': 
    main()
    pygame.quit()
